from client.client_variables import *
import datetime
import os
import sys
import logging

from sqlalchemy import create_engine, MetaData, Table, Column, Integer, String, DateTime, or_
from sqlalchemy.orm import mapper, sessionmaker
from sqlalchemy.sql import default_comparator

sys.path.append('../')


# from client_variables import *

class ClientDatabase:
    """
    the class for client database
    """
    class KnownUsers:
        """
        Database class for known users
        """

        def __init__(self, user):
            self.id = None
            self.username = user

    class MessageRegister:
        """
        the database class for message registration
        """

        def __init__(self, from_user, to_user, message_text):
            self.id = None
            self.message_from = from_user
            self.message_to = to_user
            self.message_date = datetime.datetime.now()
            self.message_text = message_text

    class Contacts:
        """the client database class for contacts"""

        def __init__(self, contact):
            self.id = None
            self.contact_name = contact

    def __init__(self, name):
        proj_path = os.path.abspath(os.getcwd())

        db_file = os.path.join(proj_path, 'client', f'client_{name}.db3')
        print(f'db_file = {db_file}')

        self.database_engine = create_engine(
            f'sqlite:////{db_file}',
            echo=False,
            pool_recycle=7200,
            connect_args={
                'check_same_thread': False})

        self.metadata = MetaData()

        known_users_table = Table('known_users', self.metadata,
                                  Column('id', Integer, primary_key=True),
                                  Column('username', String)
                                  )

        message_register_table = Table('message_history', self.metadata,
                                       Column('id', Integer, primary_key=True),
                                       Column('message_from', String),
                                       Column('message_to', String),
                                       Column('message_date', DateTime),
                                       Column('message_text', String)
                                       )

        contacts_table = Table('contacts_table', self.metadata,
                               Column('id', Integer, primary_key=True),
                               Column('contact_name', String, unique=True)
                               )

        self.metadata.create_all(self.database_engine)

        mapper(self.KnownUsers, known_users_table)
        mapper(self.MessageRegister, message_register_table)
        mapper(self.Contacts, contacts_table)

        # make session
        Session = sessionmaker(bind=self.database_engine)
        self.session = Session()

        # clear "contacts" table, since they will be downloaded from server
        self.session.query(self.Contacts).delete()
        self.session.commit()
        # clear "KnownUsers" table, since they will be downloaded from server
        self.session.query(self.KnownUsers).delete()
        self.session.commit()

    # function add contact
    def db_add_contact(self, contact):
        """
        The client database method for add contact
        :param contact:
        :return:
        """
        contact_exist: int = self.session.query(
            self.Contacts).filter_by(
            contact_name=contact).count()
        known_names = self.session.query(
            self.KnownUsers.username).filter_by(
            username=contact).count()
        if contact_exist:
            raise ValueError('Contact already exist')
        if not known_names:
            raise ValueError('Contact not Known')
        contact_row = self.Contacts(contact)
        self.session.add(contact_row)
        self.session.commit()

    def db_del_contact(self, contact):
        """
        The client database method for del single contact
        :param contact:
        :return:
        """
        test: int = self.session.query(
            self.Contacts).filter_by(
            contact_name=contact).count()
        if test:
            self.session.query(
                self.Contacts).filter_by(
                contact_name=contact).delete()
            self.session.commit()
            print(f'Contact "{contact}" deleted')

    def db_del_contacts(self):
        """
        The client database method for del all contacts
        :return:
        """
        '''Метод очищающий таблицу со списком контактов.'''
        self.session.query(self.Contacts).delete()

    def db_get_contacts(self):
        """
        The client database method to get list of contacts
        :return:
        """
        contacts_items = self.session.query(self.Contacts.contact_name).all()
        # print(f'ocntacts: {contacts_items}')
        return [contact[0] for contact in contacts_items]

    def db_check_contact(self, contact):
        """
        The client database method for checking contact
        :param contact:
        :return:
        """
        if self.session.query(
                self.Contacts).filter_by(
                contact_name=contact).count():
            return True
        else:
            return False

    def db_message_register(self, from_user, to_user, message_text):
        """
        The client database method for message registration
        :param from_user:
        :param to_user:
        :param message_text:
        :return:
        """
        message_row = self.MessageRegister(from_user, to_user, message_text)
        self.session.add(message_row)
        self.session.commit()

    def db_get_message_history(self, user=None):
        """The client database method to get message history"""
        if user:

            # msg_history = self.session.query(self.MessageRegister) \
            #     .filter(or_(self.MessageRegister.message_from == user, self.MessageRegister.message_to == user)).all()
            # return [(message.message_from, message.message_to, message.message_date, message.message_text)
            #         for message in msg_history]
            msg_history_from = self.session.query(
                self.MessageRegister).filter_by(
                message_from=user).all()
            msg_history_list1 = [
                (msg.message_from,
                 msg.message_to,
                 msg.message_date,
                 msg.message_text) for msg in msg_history_from]
            # print(f'list_from : {msg_history_list1}')
            msg_history_to = self.session.query(
                self.MessageRegister).filter_by(
                message_to=user).all()
            msg_history_list2 = [
                (msg.message_from,
                 msg.message_to,
                 msg.message_date,
                 msg.message_text) for msg in msg_history_to]
            # print(f'list_to : {msg_history_list2}')
            return msg_history_list1 + msg_history_list2

        else:
            msg_history = self.session.query(self.MessageRegister)
            return [(msg.message_from, msg.message_to, msg.message_date,
                     msg.message_text) for msg in msg_history.all()]

    def db_add_known_users(self, username):
        """
        The client database method for adding known user
        :param username:
        :return:
        """
        test = self.session.query(
            self.KnownUsers).filter_by(
            username=username).count()
        if test:
            return
        known_user_row = self.KnownUsers(username)
        self.session.add(known_user_row)
        self.session.commit()

    def db_get_known_users(self):
        """
        The client database method for get list of users
        :return:
        """
        # known_users_items = self.session.query(self.KnownUsers.username).all()
        known_users_items = self.session.query(self.KnownUsers).all()
        # return [user[0] for user in known_users_items]
        return [user.username for user in known_users_items]

    def db_check_known_user(self, user):
        """
        The client database method for checking known user
        :param user:
        :return:
        """
        if self.session.query(
                self.KnownUsers).filter_by(
                username=user).count():
            return True
        else:
            return False

    def db_del_known_users(self, user=None):
        """
        The client database method for deletion of known user
        :param user:
        :return:
        """
        if user:
            if self.session.query(
                    self.KnownUsers).filter_by(
                    username=user).first():
                self.session.query(
                    self.KnownUsers).filter_by(
                    username=user).delete()
            else:
                print(f'db_del_known_user: user {user}not found')
                return False
        else:
            self.session.query(self.KnownUsers).delete()


if __name__ == '__main__':
    test_db = ClientDatabase('user1')
    print(test_db.db_get_message_history('user1'))
"""
    # test contacts part
    for i in ['test2', 'test3', 'test4']:
        test_db.db_add_contact(i)
        print(f'db contacts : {test_db.db_get_contacts()}')
    test_db.db_add_contact('test5')
    print(f'db contacts : {test_db.db_get_contacts()}')
    print(f'db check contacts : {test_db.db_check_contact("test3")}')
    print(f'db check contacts : {test_db.db_check_contact("test6")}')

    # test message part
    test_message1 = {
        ACTION: MESSAGE,
        FROM: "test1",
        TO: "test2",
        TIME: datetime.datetime.now(),
        MESSAGE_TEXT: 'test message1  from test1 to test2'
    }
    test_message2 = {
        ACTION: MESSAGE,
        FROM: "test3",
        TO: "test1",
        TIME: datetime.datetime.now(),
        MESSAGE_TEXT: 'test message2  from test3 to test1'
    }
    test_db.db_message_register(test_message1)
    test_db.db_message_register(test_message2)
    print(f'message history : {test_db.db_get_message_history()}')

    # test KnownUsers part
    test_db.db_add_known_users("test3")
    test_db.db_add_known_users("test4")
    print(f'get_known_users : {test_db.db_get_known_users()}')
    print(f'check_known_user : {test_db.db_check_known_user("test3")}')
    print(f'check_known_user : {test_db.db_check_known_user("test5")}')
"""



# -*- coding: utf-8 -*-
from os.path import join

# Form implementation generated from reading ui file 'client_gui1.ui'
#
# Created by: PyQt5 UI code generator 5.14.1
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets
from PyQt5.QtGui import QPixmap
from PyQt5.QtWidgets import QWidget, QHBoxLayout, QLabel
import os

dir_path = os.getcwd()
logo_filename = os.path.join(dir_path, 'client' , "logo.jpg")

class Ui_MainClientWindow(object):
    def setupUi(self, MainClientWindow):
        MainClientWindow.setObjectName("MainClientWindow")
        MainClientWindow.resize(756, 534)
        MainClientWindow.setMinimumSize(QtCore.QSize(756, 534))
        self.centralwidget = QtWidgets.QWidget(MainClientWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.label_contacts = QtWidgets.QLabel(self.centralwidget)
        self.label_contacts.setGeometry(QtCore.QRect(20, 80, 101, 16))
        self.label_contacts.setObjectName("label_contacts")
        self.btn_add_contact = QtWidgets.QPushButton(self.centralwidget)
        self.btn_add_contact.setGeometry(QtCore.QRect(20, 340, 121, 31))
        self.btn_add_contact.setObjectName("btn_add_contact")
        self.btn_remove_contact = QtWidgets.QPushButton(self.centralwidget)
        self.btn_remove_contact.setGeometry(QtCore.QRect(150, 340, 121, 31))
        self.btn_remove_contact.setObjectName("btn_remove_contact")
        self.label_history = QtWidgets.QLabel(self.centralwidget)
        self.label_history.setGeometry(QtCore.QRect(310, 10, 391, 21))
        self.label_history.setObjectName("label_history")
        self.text_message = QtWidgets.QTextEdit(self.centralwidget)
        self.text_message.setGeometry(QtCore.QRect(300, 360, 441, 71))
        self.text_message.setObjectName("text_message")
        self.label_new_message = QtWidgets.QLabel(self.centralwidget)
        self.label_new_message.setGeometry(QtCore.QRect(300, 330, 250, 16))
        self.label_new_message.setObjectName("label_new_message")
        self.list_contacts = QtWidgets.QListView(self.centralwidget)
        self.list_contacts.setGeometry(QtCore.QRect(10, 110, 251, 211))
        self.list_contacts.setObjectName("list_contacts")
        self.list_messages = QtWidgets.QListView(self.centralwidget)
        self.list_messages.setGeometry(QtCore.QRect(300, 40, 441, 281))
        self.list_messages.setObjectName("list_messages")
        self.btn_send = QtWidgets.QPushButton(self.centralwidget)
        self.btn_send.setGeometry(QtCore.QRect(610, 450, 131, 31))
        self.btn_send.setObjectName("btn_send")
        self.btn_clear = QtWidgets.QPushButton(self.centralwidget)
        self.btn_clear.setGeometry(QtCore.QRect(460, 450, 131, 31))
        self.btn_clear.setObjectName("btn_clear")
        # self.label_logo = QtWidgets.QLabel(self.centralwidget)
        self.label_logo = QLabel(self.centralwidget)
        self.label_logo.setGeometry(QtCore.QRect(90, 390, 101, 81))
        pixmap = QPixmap(logo_filename)
        self.label_logo.setPixmap(pixmap)
        self.label_logo.setOpenExternalLinks(False)
        self.label_logo.setObjectName("label_logo")
        self.label_username = QtWidgets.QLabel(self.centralwidget)
        self.label_username.setGeometry(QtCore.QRect(20, 30, 201, 17))
        self.label_username.setObjectName("label_username")

        MainClientWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainClientWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 756, 25))
        self.menubar.setObjectName("menubar")
        self.menu = QtWidgets.QMenu(self.menubar)
        self.menu.setObjectName("menu")
        self.menu_2 = QtWidgets.QMenu(self.menubar)
        self.menu_2.setObjectName("menu_2")
        MainClientWindow.setMenuBar(self.menubar)

        self.statusBar = QtWidgets.QStatusBar(MainClientWindow)
        self.statusBar.setObjectName("statusBar")

        self.label_connection = QtWidgets.QLabel(self.statusBar)
        self.label_connection.setMaximumWidth(200)
        self.label_connection.setObjectName("label_connection")

        self.label_server_info = QtWidgets.QLabel(self.statusBar)
        self.label_server_info.setMaximumWidth(300)
        self.label_server_info.setObjectName("label_server_info")

        widget = QWidget()
        widget.setLayout(QHBoxLayout())
        widget.layout().addWidget(self.label_connection)
        widget.layout().addWidget(self.label_server_info)
        self.statusBar.addWidget(widget, 1)

        MainClientWindow.setStatusBar(self.statusBar)
        self.menu_exit = QtWidgets.QAction(MainClientWindow)
        self.menu_exit.setObjectName("menu_exit")
        self.menu_add_contact = QtWidgets.QAction(MainClientWindow)
        self.menu_add_contact.setObjectName("menu_add_contact")
        self.menu_del_contact = QtWidgets.QAction(MainClientWindow)
        self.menu_del_contact.setObjectName("menu_del_contact")
        self.menu.addAction(self.menu_exit)
        self.menu_2.addAction(self.menu_add_contact)
        self.menu_2.addAction(self.menu_del_contact)
        self.menu_2.addSeparator()
        self.menubar.addAction(self.menu.menuAction())
        self.menubar.addAction(self.menu_2.menuAction())

        self.retranslateUi(MainClientWindow)
        self.btn_clear.clicked.connect(self.text_message.clear)
        QtCore.QMetaObject.connectSlotsByName(MainClientWindow)

    def retranslateUi(self, MainClientWindow):
        _translate = QtCore.QCoreApplication.translate
        MainClientWindow.setWindowTitle(_translate("MainClientWindow", "Async Chat  Terminal                   BetaVersion"))
        self.label_contacts.setText(_translate("MainClientWindow", "Contacts"))
        self.btn_add_contact.setText(_translate("MainClientWindow", "Add Contact"))
        self.btn_remove_contact.setText(_translate("MainClientWindow", "Del Contact"))
        self.label_history.setText(_translate("MainClientWindow", "Messages:"))
        self.label_new_message.setText(_translate("MainClientWindow", "New message:"))
        self.btn_send.setText(_translate("MainClientWindow", "Send message"))
        self.btn_clear.setText(_translate("MainClientWindow", "Clear"))
        self.label_username.setText(_translate("MainClientWindow", "Username"))
        self.label_connection.setText(_translate("MainClientWindow", "Connection:"))
        self.label_server_info.setText(_translate("MainClientWindow", "Server Info: "))
        self.menu.setTitle(_translate("MainClientWindow", "File"))
        self.menu_2.setTitle(_translate("MainClientWindow", "Contacts"))
        self.menu_exit.setText(_translate("MainClientWindow", "Exit"))
        self.menu_add_contact.setText(_translate("MainClientWindow", "Add Contact"))
        self.menu_del_contact.setText(_translate("MainClientWindow", "Delete Contact"))



from PyQt5.QtWidgets import QDialog, QPushButton, QLineEdit, QApplication, QLabel , qApp
from PyQt5.QtCore import QEvent


# Стартовый диалог с выбором имени пользователя
class UserNameDialog(QDialog):
    def __init__(self):
        super().__init__()

        self.ok_pressed = False

        self.setWindowTitle('Привет!')
        self.setFixedSize(175, 93)

        self.label = QLabel('Введите имя пользователя:', self)
        self.label.move(10, 10)
        self.label.setFixedSize(150, 10)

        self.client_name = QLineEdit(self)
        self.client_name.setFixedSize(154, 20)
        self.client_name.move(10, 30)

        self.btn_ok = QPushButton('Начать', self)
        self.btn_ok.move(10, 60)
        self.btn_ok.clicked.connect(self.click)

        self.btn_cancel = QPushButton('Выход', self)
        self.btn_cancel.move(90, 60)
        self.btn_cancel.clicked.connect(qApp.exit)

        self.show()

    # Обработчик кнопки ОК, если поле вводе не пустое, ставим флаг и завершаем приложение.
    def click(self):
        if self.client_name.text():
            self.ok_pressed = True
            qApp.exit()


if __name__ == '__main__':
    app = QApplication([])
    dial = UserNameDialog()
    app.exec_()



### Tests for client utiils ###

import os
import sys
import unittest

sys.path.append(os.path.join(os.getcwd(), '../Lesson3'))
from client.client_utils import get_ip_address, presence_msg, process_ans, client_connection, user_msg


class TestClass(unittest.TestCase):

    def test_command_line_address_1(self):
        commandline = ['-a', '127.0.0.1', '-p', '7777']
        self.assertEqual(get_ip_address(commandline), '127.0.0.1')

    # Это не работает, в функции стоит sys.exit(1), ругается на эту команду

    def test_command_line_address_2(self):
        commandline = ['-a']
        with self.assertRaises(SystemExit):
            get_ip_address(commandline)

    def test_command_line_address_3(self):
        commandline = []
        self.assertEqual(get_ip_address(commandline), None)

    def test_presence_msg_1(self):
        result ={'action': 'presence', 'time': 0.001, 'type': 'status',
                   'user': {"account_name": 'Guest', "status": "connected" }}
        test = presence_msg()
        test['time'] = 0.001
        self.assertEqual(test, result)

    def test_presence_msg_2(self):
        result ={'action': 'presence', 'time': 0.001, 'type': 'status',
                   'user': {"account_name": 'User', "status": "connected" }}
        test = presence_msg('User')
        test['time'] = 0.001
        self.assertEqual(test, result)

    def test_process_ans_1(self):
        message = {'answer': 200}
        with self.assertRaises(ValueError):
            process_ans(message)

    def test_process_ans_2(self):
        message = {'response': "ABC"}
        with self.assertRaises(ValueError):
            process_ans(message)

    def test_process_ans_200(self):
        message = {'response': 200}
        self.assertEqual(process_ans(message), '200: OK')

    def test_process_ans_400(self):
        message = {'response': 400}
        self.assertEqual(process_ans(message), '400: Error')

    def test_client_connection_if_server_down(self):
        server_ip = '127.0.0.1'
        server_port = 7777
        with self.assertRaises(SystemExit):
            client_connection(server_ip, server_port)


    def test_user_message_1(self):
        print('\n test "user_msg:"')
        result = user_msg()
        # {'action': "msg", 'time': time.time(), 'to': user_to, 'from': 'Guest',
        #  'encoding': ENCODING, 'message': user_message}
        test = "action" in result \
            and result["action"] == 'msg'  \
            and "to" in result  \
            and isinstance(result['to'], str) \
            and "from" in result \
            and isinstance(result['from'], str) \
            and "encoding" in result \
            and "message" in result \
            and isinstance(result['message'], str)
        self.assertTrue(test)


if __name__ == '__main__':
    unittest.main()



# client_log_config #

import sys
import os
import logging.handlers
import logging

sys.path.append('../Lesson3/')
from client.client_variables import FILE_LOGGING_LEVEL, TERMINAL_LOGGING_LEVEL


# formatting depending on logging.LEVEL below:


LOG_FILE_PATH = os.getcwd()
LOG_FILE_NAME = os.path.join(LOG_FILE_PATH, 'log', 'logs', 'client', 'client.log')

LOGGER = logging.getLogger('client_logger')

LOGGER_FORMATTER = logging.Formatter('%(asctime)-16s %(levelname)-10s %(filename)-22s %(module)-14s %(funcName)s %(message)s')


STREAM_HANDLER = logging.StreamHandler(sys.stderr)
STREAM_HANDLER.setFormatter(LOGGER_FORMATTER)
STREAM_HANDLER.setLevel(TERMINAL_LOGGING_LEVEL)

FILE_HANDLER = logging.FileHandler(LOG_FILE_NAME, encoding='utf8')

# FILE_HANDLER = logging.handlers.TimedRotatingFileHandler(LOG_FILE_NAME,
#                                                          encoding = 'UTF-8',  interval=1, when='M',
#                                                          backupCount=10, delay = True)
FILE_HANDLER.setFormatter(LOGGER_FORMATTER)
FILE_HANDLER.setLevel(FILE_LOGGING_LEVEL)

LOGGER.addHandler(STREAM_HANDLER)
LOGGER.addHandler(FILE_HANDLER)
LOGGER.setLevel(logging.DEBUG)


if __name__ == '__main__':
    LOGGER.debug('test debug message')
    LOGGER.info('test info message')
    LOGGER.error('test error message')
    LOGGER.critical('test critical message')


""" Constants """

import logging

DEFAULT_CLIENT_PORT = 7777
DEFAULT_CLIENT_IP_ADDRESS = '127.0.0.1'
# Максимальная очередь подключений
MAX_CONNECTIONS = 5
# Максимальная длинна сообщения в байтах
MAX_PACKAGE_LENGTH = 1024
# Кодировка проекта
ENCODING = 'utf-8'

# Прококол JIM основные ключи:
ACTION = 'action'
TIME = 'time'
MESSAGE = 'message'
MESSAGE_TEXT = 'message_text'
SENDER = 'sender'
FROM = 'from'
TO = 'to'
EXIT = 'exit'

# Прочие ключи, используемые в протоколе
PRESENCE = 'presence'
RESPONSE = 'response'
ERROR = 'error'
STATUS = 'status'
TYPE = 'type'
PUBLIC_KEY = 'public_key'

# Logging
# Logging levels
FILE_LOGGING_LEVEL = logging.DEBUG
TERMINAL_LOGGING_LEVEL = logging.DEBUG
GET_CONTACTS = 'get_contacts'
ADD_CONTACT = 'add_contact'
CONTACT = 'contact'
USERS_REQUEST = 'users_request'
REMOVE_CONTACT = 'remove_contact'
DATA_LIST = 'data_list'
DATA = 'data'


RESPONSE_200 = {'action': 'response', 'response': 200}
RESPONSE_202 = {'action': 'response', 'response': 202}
# 400
RESPONSE_400 = {'action': 'response', 'response': 400, 'error': 'bad request'}
RESPONSE_511 = {'action': 'response', RESPONSE: 511}




### Tests for client utiils ###

import os
import sys
import unittest

sys.path.append(os.path.join(os.getcwd(), '../Lesson3'))
from server.server_utils import get_ip_address, process_incoming_message
from server.server_variables import RESPONSE, ACTION, PRESENCE, TIME


class TestClass(unittest.TestCase):

    def test_command_line_address_1(self):
        commandline = ['-a', '127.0.0.1', '-p', '7777']
        self.assertEqual(get_ip_address(commandline), '127.0.0.1')

    # Это не работает, в функции стоит sys.exit(1), ругается на эту команду

    def test_command_line_address_2(self):
        commandline = ['-a']
        with self.assertRaises(SystemExit):
            get_ip_address(commandline)

    def test_command_line_address_3(self):
        commandline = []
        self.assertEqual(get_ip_address(commandline), None)

    def test_process_incoming_message_1(self):
        test_message = {ACTION: PRESENCE,
                        TIME: 0.1,
                        'user':
                            {'account_name': 'Guest'}
                        }
        test_response = {'response': {RESPONSE: 200}}
        self.assertEqual(process_incoming_message(test_message), test_response)

    def test_process_incoming_message_2(self):
        test_message = {ACTION: 'msg',
                        TIME: 0.1,
                        'to': 'user2',
                        'from': 'Guest'
                        }
        test_response = {'response': {RESPONSE: 201}}
        self.assertEqual(process_incoming_message(test_message), test_response)

    def test_process_incoming_message_3(self):
        test_message = {ACTION: 'quit'}
        test_response = {'response': {RESPONSE: 202}, 'quit': " "}
        self.assertEqual(process_incoming_message(test_message), test_response)

    def test_process_incoming_message_4(self):
        test_message = {ACTION: "something",
                        TIME: 0.1,
                        'user':
                            {'account_name': 'Guest'}
                        }
        test_response = {'response': {'response': 400,
                                      'error': 'Bad Request'
                                      }}
        self.assertEqual(process_incoming_message(test_message), test_response)

    def test_process_incoming_message_5(self):
        test_message = {ACTION: PRESENCE,
                        "times": 0.1,
                        'user':
                            {'account_name': 'Guest'}
                        }
        test_response = {'response': {'response': 400,
                                      'error': 'Bad Request'
                                      }}
        self.assertEqual(process_incoming_message(test_message), test_response)

    def test_process_incoming_message_6(self):
        test_message = {ACTION: PRESENCE,
                        TIME: 0.1,
                        'user':
                            {'account_name': 'USER'}
                        }
        test_response = {'response': {'response': 400,
                                      'error': 'Bad Request'
                                      }}
        self.assertEqual(process_incoming_message(test_message), test_response)





    def test_process_incoming_message_6(self):
        test_message = "something_else"
        test_response = {'response':
                             {'response': 400,
                              'error': 'Bad Request'
                              }}
        self.assertEqual(process_incoming_message(test_message), test_response)





if __name__ == "__main__":
    unittest.main()



import sys
import os
import unittest
import json
sys.path.append(os.path.join(os.getcwd(), '../Lesson3'))

from server.server_utils import get_message, send_message


class Test_Sock():
    """
    Класс эмулирует поведение Socket, включая функции recv и send
    """
    def __init__(self):
        self.sock_sent_msg = None
        self.sock_received_msg = {'test': "test message"}

    def recv(self, packet_length):
        """
        Функция Позволяет тестировать поведение функций, включающих Socket.recv
        берет self.sock_received_msg ,
        делает > json.dumps > bynary string   ( Эмуляция работы функции Socket.recv )
        :param packet_length: max TCP package length;
        :return Bynary string:
        """
        json_obj = json.dumps(self.sock_received_msg)
        return json_obj.encode('utf-8')

    def send(self, byte_string):
        """
        Функция Позволяет тестировать поведение функций, включающих Socket.send
        берет byte_string ,
        восстанавливает > decode('utf-8) > json.loads
        записывает в переменную self.sock_sent_msg словарь , получающийся при декодировании Byte_string.
        :param byte_string:
        :return: None
        """
        json_string = byte_string.decode('utf-8')
        self.sock_sent_msg = json.loads(json_string)


class TestSendRcv(unittest.TestCase):

    def test_send_msg(self):
        """
        функция тестирования функции send_message
        Класс Test_Sock определяет функцию sock.send,
        которая декодирует передаваемое сообщение  в предполагаемый словарь
        записывает пердполагаемый передаваемый словарь в переменную self.sock_sent_msg
        далее сравнивеат переданный словарь "test_message" c декодированным словарем полученным из self.send
        :return:
        """
        sock = Test_Sock()
        test_message = {'response': 200}
        send_message(sock, test_message)
        self.assertEqual(sock.sock_sent_msg, test_message)

    def test_get_message(self):
        """
        функция тестирования функции get_message
        Класс Test_Sock определяет функцию self.recv,
        Функция get_message
        вызывает функцию  self.recv, которая врзвращает бинарнусю строку , соответсвующую тестовому словарю test_dict
        далее сравниваем рузельтат функции get_message и тестовый словарь
        :return:
        """
        sock = Test_Sock()
        test_dict = get_message(sock)
        self.assertEqual(sock.sock_received_msg, test_dict)


if __name__ == '__main__':
    unittest.main()












import json
import sys
import subprocess
import socket

sys.path.append('../')
import log.server_log_config
from common.errors import NonDictInputError, IncorrectDataRecivedError, JSONDecodeError
from server.server_variables import *
from server.server_decos import log

# Инициализация логирования сервера.
LOGGER = logging.getLogger('server_logger')


@log
def get_message(client):
    """
    Утилита приёма и декодирования сообщения принимает байты выдаёт словарь,
    если приняточто-то другое отдаёт ошибку значения
    :param client:
    :return:
    """
    message_encoded = client.recv(MAX_PACKAGE_LENGTH)
    if isinstance(message_encoded, bytes):
        try:
            # print(f'          trying decode {message_encoded}')
            message_decoded = message_encoded.decode(ENCODING)
            # print(f'          trying make dict from {message_decoded}')
            message_dict = json.loads(message_decoded)
            # print(f'          dict composed {message_dict}')
        except JSONDecodeError:
            # print(f' got message {message_encoded}, fail to decode the message')
            raise JSONDecodeError
            # return
        if isinstance(message_dict, dict):
            print(f'got message {message_dict} ')
            return message_dict
        else:
            raise IncorrectDataRecivedError('msg is not dict')
    else:
        raise IncorrectDataRecivedError('incoming msg is not bytes')


@log
def send_message(sock, message):
    """
    Утилита кодирования и отправки сообщения
    принимает словарь и отправляет его
    :param sock:
    :param message:
    :return:
    """
    print(f'sending message {message}  to {sock}')
    if not isinstance(message, dict):
        raise NonDictInputError
    js_message = json.dumps(message)
    encoded_message = js_message.encode(ENCODING)
    sock.send(encoded_message)


def pid_used_port(port_numb):
    """
    the function indicates number of port which occupies port 7777
    :param port_numb:
    :return:
    """
    params = str()
    cmd = ['netstat', '-ntlp']
    param = []
    subproc = subprocess.Popen(cmd, stdout=subprocess.PIPE)
    while True:
        line = subproc.stdout.readline()
        decoded_line = line.decode('ASCII')
        if str(port_numb) in decoded_line:
            params = decoded_line.split()
            param = params[6].split('/')
            # print(f' decoded_line: {decoded_line}')
            # print(f' param[0] : {param[0]}')
        if not line:
            break
        subproc.terminate()
        print(f'port {port_numb}  is in use, process : {params}')
    try:
        return param[0]
    except IndexError:
        return


def print_cli_help():
    print('Поддерживаемые комманды:')
    print('users - список известных пользователей')
    print('connected - список подключенных пользователей')
    print('loghist - история входов пользователя')
    print('exit - завершение работы сервера.')
    print('help - вывод справки по поддерживаемым командам')


if __name__ == '__main__':
    print(f'pid = {pid_used_port(7777)}')















import logging
import sys

logger = logging.getLogger('server')


class Port:
    def __set__(self, instance, value):
        if not 1023 < value < 65536:
            logger.criticalc(f'incorrect port {value} number, program exit')
            print(f'incorrect port {value} number, program exit')
            sys.exit(1)
        instance.__dict__[self.name] = value

    def __set_name__(self, owner, name):
        self.name = name



from setuptools import setup, find_packages

setup(name='msnger_client',
      version="1.0.0",
      description='msnger_client',
      author='Vladimir Malchevskiy',
      author_email='vladimir.malchevskiy@gmail.com',
      packages=find_packages(),
      install_requires=['PyQt5', 'sqlalchemy', 'pycryptodome', 'pycryptodomex']
      )
